/*
 * $Id: RCCommunicator.java,v 1.20 2002/07/05 18:34:08 stork Exp $
 *
 * $Log: RCCommunicator.java,v $
 * Revision 1.20  2002/07/05 18:34:08  stork
 * update dokumentation
 *
 * Revision 1.19  2002/07/04 16:28:45  stork
 * input buffer to 2kB
 *
 * Revision 1.17  2002/07/04 16:19:13  stork
 * reactivate log comments
 *
 * Revision 1.15  2002/07/04 14:00:46  stork
 * remove send buffer size
 *
 * Revision 1.14  2002/07/04 13:48:07  transier
 * save last sent command
 *
 * Revision 1.13  2002/07/04 13:37:05  stork
 * set send buffer to 0
 *
 * Revision 1.11  2002/07/04 12:54:45  stork
 * new buffer size
 *
 * Revision 1.10  2002/07/03 17:22:48  stork
 * +fix too small buffer for server messages
 * +add some debug messages
 *
 * Revision 1.9  2002/06/27 20:26:39  stork
 * add + update dokumentation
 *
 * Revision 1.8  2002/06/22 20:15:41  stork
 * add some more dokumentation
 *
 * Revision 1.7  2002/06/08 14:39:20  stork
 * add log support
 *
 * Revision 1.6  2002/06/07 18:13:42  stork
 * remove comments
 *
 * Revision 1.5  2002/05/30 15:37:16  stork
 * improve handling when server dies before we
 *
 * Revision 1.4  2002/05/29 21:05:21  stork
 * update say methode
 *
 * Revision 1.3  2002/05/25 22:05:12  stork
 * tests and minor bugfixes
 *
 * Revision 1.2  2002/05/21 20:57:14  stork
 * first communicator version
 *
 * Revision 1.1.1.1  2002/05/21 20:32:05  stork
 * empty project
 *
 */
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * This class id for the connection to the server. It log on to the server
 * and receive the messages from the server. It use the RCParser class for 
 * parsing and delivering the servermessages.
 */
public class RCCommunicator 
    implements RCEventType, RCSender,RCType, RCView, RCPlayMode {
    /** Attributes */
    public String Host;
    public int ServerPort;
    public String Team;
    public int PlayerNumber;
    public RCPlayMode PlayMode;
    public boolean Goalie;
    public DatagramSocket Sock;  
    public InetAddress Address;
    /** Associations */
    private RCParser Parser;
    private RCWorld World;

    /**
     * Constructor for RCCommunicator oject.
     *
     * @param Team name of the team
     * @param World referenz to the world object
     * @param Goalie are we a goalie
     * @param Host name of the server hostname
     * @param Port port of the server
     */
    public RCCommunicator ( String Team, RCWorld World, boolean Goalie, String Host, int Port ){
	String InitMsg;
	
	/* create a new parser */
	Parser = new RCParser(Team);
	this.Team = Team;
	this.Goalie = Goalie;
	this.Host = Host;
	this.ServerPort = Port;
	this.World = World;
	/* do the network stuff*/
	try{
	    /* try to resolve the Hostname */
	    Address = InetAddress.getByName(Host);
	    /* create a new socket */
	    Sock = new DatagramSocket();
	    Sock.setSoTimeout(10000);

	    /* "connect" to server */
	    if( Goalie == false )
		InitMsg = "(init "+Team+"(version 7.0))";
	    else
		InitMsg = "(init "+Team+"(version 7.0) (goalie))";
	    this.send(InitMsg);
	  
	    // wait for response from server
	    byte[] Response = new byte[4096];
	    DatagramPacket rp = new DatagramPacket(Response, Response.length);  
	    Sock.receive(rp);	    /* save server port */
	    ServerPort = rp.getPort();
	    String ResponseMsg = new String(rp.getData());

	    if( ResponseMsg.startsWith("(error") == true ){
		// shit happens
		System.out.println(ResponseMsg);
		System.exit(1);
	    }else{
		/* analyse the server response */
		String Message = ResponseMsg.substring(6, ResponseMsg.length()-1);
		StringTokenizer tk = new StringTokenizer(Message);
		if( tk.nextToken().compareTo("l") == 0 ){
		    // left to right
		    World.Side = LEFT;
		}else{
		    // right to left
		    World.Side = RIGHT;
		}
		this.PlayerNumber = Integer.parseInt(tk.nextToken());
		World.PlayerNumber = this.PlayerNumber;
	    }
	    
	    // set position
	    this.move((World.X1+World.X2)/2,
		      (World.Y1+World.Y2)/2);
	    
	}catch(Exception e){
	    World.Log.Log("Player-"+World.PlayerNumber+" : Resolve error "+e.toString());
	    System.out.println("RESOLVE:"+ e.toString());
	}
    }
    
    /**
     * receive a package, and do a cycle. If there was an error 
     * it returns false (example server closed);
     *
     * @return true on success else false 
     */
    public boolean receive (  ){

	try{
	    byte[] data = new byte[2048];
	    /* get new message */
	    DatagramPacket p = new DatagramPacket(data, data.length); 
	    this.Sock.receive(p);
	    /* parse the message */
	    Parser.pushMessage(new String(p.getData()), World);
	}catch(Exception e){
	    //System.out.println(e.toString());
	    //	    System.exit(1);	    
	    return false;
	}
	return true;
    }

    /**
     * send a move message to the server
     *
     * @param Movement angle of movement
     */
    public void turn ( double Movement ){
	this.send("(turn "+Movement+")");
    }

    /**
     * send a dash message to the server
     *
     * @param Power power of the dash
     */
    public void dash ( double Power ){
	this.send("(dash "+Power+")");
    }

    /**
     * send a kick message to the server
     *
     * @param Power power of the kick
     * @param Direction direction of the kick
     */
    public void kick ( double Power, double Direction ){
	this.send("(kick "+Power+" "+Direction+")");
    }

    /**
     * send a catch message to the server
     *
     * @param Direction the direction to catch
     */
    public void catchit ( double Direction ){
	this.send("(catch "+Direction+")");
    }
    
    /**
     * send a move message to the server
     *
     * @param x x-coordinate
     * @param y y-coordinate
     */
    public void move ( double x, double y ){
	this.send("(move "+x+" "+y+")");
    }

    /**
     * Send a turn_neck message to the sever.
     *
     * @param Angle angle to turn 
     */
    public void turn_neck ( double Angle ){
	this.send("(turn_neck "+Angle+")");
    }

    /**
     * Send a say message to the server, with World.MsgStart as prefix.
     *
     * @param Message
     * @return boolean
     */
    public void say ( String Message ){
	this.send("("+World.MsgStart+"say "+Message+")");
    }

    /**
     * Send a body_sence message to the server.
     */
    public void sence_body (  ){
	this.send("(sence_body)");
    }

    /**
     * This function sends a message to the server. All other send-methods
     * are calling this function to deliver their messages.
     *
     * @param Message message to send
     */
    public void send(String Message){
	/* create a new packet */
	DatagramPacket p = new DatagramPacket(Message.getBytes(), Message.length());
	p.setPort(this.ServerPort);
	p.setAddress(this.Address);
	try{
	    /* send a new packet */
	    Sock.send(p);
	}catch(IOException e){
	    System.out.println(e.toString());
	    System.exit(1);
	}
    }
}

