/*
 * $Id: RCGoalieBrain.java,v 1.8 2002/07/09 21:42:32 stork Exp $
 *
 * $Log: RCGoalieBrain.java,v $
 * Revision 1.8  2002/07/09 21:42:32  stork
 * update doukumentation
 *
 * Revision 1.7  2002/07/07 22:42:39  transier
 * minimal final changes
 *
 * Revision 1.6  2002/07/07 19:31:44  ritter
 * final version of goalie
 *
 * Revision 1.5  2002/07/07 19:11:39  ritter
 * second versinon of goalie
 *
 * Revision 1.4  2002/07/07 15:06:32  transier
 * debugging
 *
 * Revision 1.3  2002/07/07 14:10:39  ritter
 * first version of goalie
 *
 * Revision 1.2  2002/07/07 11:24:47  transier
 * import call removed
 *
 * Revision 1.1  2002/07/06 13:20:50  transier
 * empty Brain for Goalie
 *
 *
 */

/**
 * the strategy for goalies
 */
public class RCGoalieBrain extends RCBrain {

  /** Associations */
  private final double MAX_DIST =1.;
  protected final double HAS_BALL = .5;	     // defines threshold for "a player has the ball"
  protected final int MAX_TIME = 3;	// defines threshold in which position information of objects are valid
  protected final double MIN_POWER_DASH = 1;	// defines threshold for the minimum power a dash command is submitted
  protected final double MAX_ANGLE = .5;	// defines threshold for the maximunm angle differece between the player and his target point
  protected final double POWER_COEFF_DASH = 20.;	// defines relation between distance and power for dash command
  protected final double NEAR_BALL = .6;	// defines threshold for "the ball is near"
  
  /**
   * every cycle called by the main loop
   *
   */
  public int update(){
  	
  	RCObject me = World.getOurself ();
    myX = me.X (0);
    myY = me.Y (0);

    double ballX = World.Ball.X (0);
    double ballY = World.Ball.Y (0);
    double ballDist = calcDist (myX, myY, ballX, ballY);
    double angle;
    
    // found ball
	if (searchBall()){
		
	    // goalie has ball
		if (ballDist <= HAS_BALL){
		  angle = getAngleToPoint(0, 0);
		  Communicator.kick(100, angle);
		  return 0;
		}
		
		// ball is in goalie's field and he hasn't got it yet
		else if (inField (ballX, ballY)){			
		  // turn to ball
          angle = getAngleToPoint(ballX, ballY);
          if (Math.abs (angle) >= MAX_ANGLE && !World.NewData){
    	    Communicator.turn (angle);
		    return 0;
          }
          // run to ball
	      if (Math.abs (ballDist) >= NEAR_BALL){
	        Communicator.dash (dashPower(ballDist));
		    return 0;
          }
          // goalie is NEAR_BALL => catch
          else {
            angle = getAngleToPoint(ballX, ballY);
            Communicator.catchit(angle);       
            return 0;
          }       
      }  
      
      
      // ball isn't in goalie's field
      else if (inField (myX, myY)){

		// get destination point
		double x = -GOAL_X + 1.5;
		double y = 0; //ballX*(14/68);
		// turn to destination point
		double dist = calcDist(x, y, myX, myY);
		angle = getAngleToPoint(x, y);
        if ((Math.abs (angle) >= MAX_ANGLE && !World.NewData) && (Math.abs (dist) >= MAX_DIST)){
   	  
    	  Communicator.turn (angle);
		  return 0;
        }
        // go to destination point
	    if (Math.abs (dist) >= MAX_DIST){
	    	
	      Communicator.dash (dashPower(dist)*0.5);
		  return 0;
        }
        // turn to ball
        angle = getAngleToPoint(ballX, ballY);
        if (Math.abs (angle) >= MAX_ANGLE && !World.NewData){
    	  Communicator.turn (angle);   	  
		  return 0;
        }
        return 0;  
      }  
      
      // ball and goalie aren't in goalie's field  
	  else{    
	    // turn to middel of goal
	    angle = getAngleToPoint(-GOAL_X + 1.5, -GOAL_Y);
        if (Math.abs (angle) >= MAX_ANGLE && !World.NewData){
    	  Communicator.turn (angle);
		  return 0;
        }
        // go to the middle of the goal
        double dist = calcDist(-GOAL_X + 1.5, -GOAL_Y, myX, myY);
	    if (Math.abs (dist) >= MAX_DIST){
	      Communicator.dash (dashPower(dist));
		  return 0;
        }
        return 0; 
	  }
	}
	// Don't know where the ball is  	
	else{
        if (inField (myX, myY)){
	      // get destination point
		  double x = -GOAL_X + 1.5;
		  double y = 0; //ballX*(14/68);
		  // turn to destination point
		  double dist = calcDist(x, y, myX, myY);
		  angle = getAngleToPoint(x, y);
          if ((Math.abs (angle) >= MAX_ANGLE && !World.NewData) && (Math.abs (dist) >= MAX_DIST)){   	  
    	    Communicator.turn (angle);
		    return 0;
          }
          // go to destination point
	      if (Math.abs (dist) >= MAX_DIST){  	
	        Communicator.dash (dashPower(dist)*0.5);
		    return 0;
          }
       }  
        else{
          angle = getAngleToPoint(-GOAL_X + 1.5, -GOAL_Y);
          if (Math.abs (angle) >= MAX_ANGLE && !World.NewData){
    	    Communicator.turn (angle);
		    return 0;
          }
          // go to the middle of the goal
          double dist = calcDist(-GOAL_X + 1.5, -GOAL_Y, myX, myY);
	      if (Math.abs (dist) >= MAX_DIST){
	        Communicator.dash (dashPower(dist));
		    return 0;
          }
        }			
      // turn to see the ball
	  if (!World.NewData){
	    Communicator.turn (SEARCH_ANGLE);
		return 0;
	  }
	  else
	  return 0;	
    } 
  }

  /**
   * constructor of RCGoalieBrain
   *
   * @param World
   * @param Communicator
   */
  public RCGoalieBrain (RCWorld World, RCCommunicator Communicator)
  {
  	super(World, Communicator);
  }
	
}
