/*
 * $Id: RCParser.java,v 1.15 2002/07/05 18:34:08 stork Exp $
 *
 * $Log: RCParser.java,v $
 * Revision 1.15  2002/07/05 18:34:08  stork
 * update dokumentation
 *
 * Revision 1.14  2002/07/03 19:06:01  stork
 * enable output of the server messages to the log file
 *
 * Revision 1.13  2002/07/03 17:23:15  stork
 * add some debug messages
 *
 * Revision 1.12  2002/06/27 20:26:39  stork
 * add + update dokumentation
 *
 * Revision 1.11  2002/06/22 20:15:41  stork
 * add some more dokumentation
 *
 * Revision 1.10  2002/06/10 20:52:19  stork
 * correct handling of enemies and friend
 *
 * Revision 1.9  2002/06/08 14:40:33  stork
 * add logging support
 *
 * Revision 1.8  2002/06/07 21:36:17  stork
 * syncronize and improve the event parsing with the world
 *
 * Revision 1.7  2002/06/07 18:12:51  stork
 * bug fix in flag handling
 *
 * Revision 1.5  2002/06/04 16:39:02  stork
 * correct bug in sorting list
 *
 * Revision 1.4  2002/05/29 21:06:18  stork
 * add missing Hear type value
 *
 * Revision 1.3  2002/05/25 22:05:12  stork
 * tests and minor bugfixes
 *
 * Revision 1.2  2002/05/21 20:57:14  stork
 * first communicator version
 *
 * Revision 1.1.1.1  2002/05/21 20:32:05  stork
 * empty project
 *
 */
import java.util.*;

/**
 * This class parse all messages from the server. It creates a
 * list of the corresponding events the will be delivered to the
 * World object.
 */
public class RCParser
    implements RCEventType, RCSender,RCType, RCView, RCPlayMode {

    // String data base
    private final String ServerParam = "(server_param ";
    private final String PlayerType = "(player_type ";
    private final String PlayerParam = "(player_param ";
    private final String Hear = "(hear ";
    private final String See = "(see ";
    private final String SenceBody = "(sense_body ";
    
    private String TeamName;   /**< Teamname */

    int OldTimeStamp;

    /**
     * Constructor.
     */
    public RCParser(String TeamName){
	this.TeamName = TeamName;
	this.OldTimeStamp = 0;
    }

    /**
     * Operation
     *
     * @param Command message received from server
     * @param World ref. to world
     */
    public void pushMessage ( String Command, RCWorld World ){
	String Message = Command.trim();

	/* identify messsage type */
	if( Message.startsWith(ServerParam) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse ServerParam-Message");
	    this.parseServerParam(Message, World);
	}else if( Message.startsWith(PlayerType) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse PlayerType-Message");
	    this.parsePlayerType(Message, World);
	}else if( Message.startsWith(PlayerParam) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse PlayerParam-Message");
	    this.parsePlayerParam(Message, World);
	}else if( Message.startsWith(Hear) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse Hear-Message");
	    this.parseHear(Message, World);
	}else if( Message.startsWith(See) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse See-mMessage");
	    this.parseSee(Message, World);
	}else if( Message.startsWith(SenceBody) == true ){
	    World.Log.Log("Player-"+World.PlayerNumber+" : parse SenceBody-Message");
	    this.parseSenseBody(Message, World);
	}
    }


    /**
     * Parse a server param message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parseServerParam(String Message, RCWorld World){
	RCServerParamEvent event = new RCServerParamEvent();
	String data = Message.substring( ServerParam.length(), Message.length()-1);
	StringTokenizer tk = new StringTokenizer(data);

	/* parse the message */
	event.Type = SERVER_PARAM;	
	event.gwidth = Double.parseDouble(tk.nextToken());
	event.inertia_moment = Double.parseDouble(tk.nextToken());
	event.psize = Double.parseDouble(tk.nextToken());
	event.pdecay = Double.parseDouble(tk.nextToken());
	event.prand = Double.parseDouble(tk.nextToken());
	event.pweight = Double.parseDouble(tk.nextToken());
	event.pspeed_max = Double.parseDouble(tk.nextToken());
	event.paccel_max = Double.parseDouble(tk.nextToken());
	event.stamia_max = Double.parseDouble(tk.nextToken());
	event.stamia_inc = Double.parseDouble(tk.nextToken());
	event.recover_init = Double.parseDouble(tk.nextToken());
	event.recover_dthr = Double.parseDouble(tk.nextToken());
	event.recover_min = Double.parseDouble(tk.nextToken());
	event.recover_dec = Double.parseDouble(tk.nextToken());
	event.effort_init = Double.parseDouble(tk.nextToken());
	event.effort_dthr = Double.parseDouble(tk.nextToken());
	event.effor_min = Double.parseDouble(tk.nextToken());
	event.effort_dec = Double.parseDouble(tk.nextToken());
	event.effort_ithr = Double.parseDouble(tk.nextToken());
	event.effort_inc = Double.parseDouble(tk.nextToken());
	event.kick_rand = Double.parseDouble(tk.nextToken());
	event.team_actuator_noise = Double.parseDouble(tk.nextToken());
	event.prand_factor_l = Double.parseDouble(tk.nextToken());
	event.prand_factor_r = Double.parseDouble(tk.nextToken());
	event.kick_rand_factor_l = Double.parseDouble(tk.nextToken());
	event.kick_rand_factor_r = Double.parseDouble(tk.nextToken());
	event.bsize = Double.parseDouble(tk.nextToken());
	event.bdecay = Double.parseDouble(tk.nextToken());
	event.brand = Double.parseDouble(tk.nextToken());
	event.bweight = Double.parseDouble(tk.nextToken());
	event.bspeed_max = Double.parseDouble(tk.nextToken());
	event.baccel_max = Double.parseDouble(tk.nextToken());
	event.dprate = Double.parseDouble(tk.nextToken());
	event.kprate = Double.parseDouble(tk.nextToken());
	event.kmargin = Double.parseDouble(tk.nextToken());
	event.ctlradius = Double.parseDouble(tk.nextToken());
	event.ctlradius_width = Double.parseDouble(tk.nextToken());
	event.maxp = Double.parseDouble(tk.nextToken());
	event.minp = Double.parseDouble(tk.nextToken());
	event.maxm = Double.parseDouble(tk.nextToken());
	event.minm = Double.parseDouble(tk.nextToken());
	event.maxnm = Double.parseDouble(tk.nextToken());
	event.minnm = Double.parseDouble(tk.nextToken());
	event.maxn = Double.parseDouble(tk.nextToken());
	event.minn = Double.parseDouble(tk.nextToken());
	event.visangle = Double.parseDouble(tk.nextToken());
	event.visdist = Double.parseDouble(tk.nextToken());
	event.windir = Double.parseDouble(tk.nextToken());
	event.winforce = Double.parseDouble(tk.nextToken());
	event.winang = Double.parseDouble(tk.nextToken());
	event.winrand = Double.parseDouble(tk.nextToken());
	event.kickable_area = Double.parseDouble(tk.nextToken());
	event.catch_area_l = Double.parseDouble(tk.nextToken());
	event.catch_area_w = Double.parseDouble(tk.nextToken());
	event.catch_prob = Double.parseDouble(tk.nextToken());
	event.goalie_max_moves = Double.parseDouble(tk.nextToken());
	event.ckmargin = Double.parseDouble(tk.nextToken());
	event.offside_area = Double.parseDouble(tk.nextToken());
	event.win_no = Double.parseDouble(tk.nextToken());
	event.win_random = Double.parseDouble(tk.nextToken());
	event.say_cnt_max = Double.parseDouble(tk.nextToken());
	event.SayCoachMsgSize = Double.parseDouble(tk.nextToken());
	event.clang_win_size = Double.parseDouble(tk.nextToken());
	event.clang_define_win = Double.parseDouble(tk.nextToken());
	event.clang_meta_win = Double.parseDouble(tk.nextToken());
	event.clang_advice_win = Double.parseDouble(tk.nextToken());
	event.clang_info_win = Double.parseDouble(tk.nextToken());
	event.clang_mess_delay = Double.parseDouble(tk.nextToken());
	event.clang_mess_per_cycle = Double.parseDouble(tk.nextToken());
	event.half_time = Double.parseDouble(tk.nextToken());
	event.sim_st = Double.parseDouble(tk.nextToken());
	event.send_st = Double.parseDouble(tk.nextToken());
	event.recv_st = Double.parseDouble(tk.nextToken());
	event.sb_step = Double.parseDouble(tk.nextToken());
	event.lcm_st = Double.parseDouble(tk.nextToken());
	event.SayMsgSize = Double.parseDouble(tk.nextToken());
	event.hear_max = Double.parseDouble(tk.nextToken());
	event.hear_inc = Double.parseDouble(tk.nextToken());
	event.hear_decay = Double.parseDouble(tk.nextToken());
	event.cban_cycle = Double.parseDouble(tk.nextToken());
	event.slow_down_factor = Double.parseDouble(tk.nextToken());
	event.useoffside = Double.parseDouble(tk.nextToken());
	event.kickoffoffside = Double.parseDouble(tk.nextToken());
	event.offside_kick_margin = Double.parseDouble(tk.nextToken());
	event.audio_dist = Double.parseDouble(tk.nextToken());
	event.dist_qstep = Double.parseDouble(tk.nextToken());
	event.land_qstep = Double.parseDouble(tk.nextToken());
	event.dir_qstep = Double.parseDouble(tk.nextToken());
	event.dist_qstep_l = Double.parseDouble(tk.nextToken());
	event.dist_qstep_r = Double.parseDouble(tk.nextToken());
	event.land_qstep_l = Double.parseDouble(tk.nextToken());
	event.land_qstep_r = Double.parseDouble(tk.nextToken());
	event.dir_qstep_l = Double.parseDouble(tk.nextToken());
	event.dir_qstep_r = Double.parseDouble(tk.nextToken());
	event.CoachMode = Double.parseDouble(tk.nextToken());
	event.CwRMode = Double.parseDouble(tk.nextToken());
	event.old_hear = Double.parseDouble(tk.nextToken());
	event.sv_st = Double.parseDouble(tk.nextToken());
	event.start_goal_l = Double.parseDouble(tk.nextToken());
	event.start_goal_r = Double.parseDouble(tk.nextToken());
	event.fullstate_l = Double.parseDouble(tk.nextToken());
	event.fullstate_r = Double.parseDouble(tk.nextToken());
	event.drop_time = Double.parseDouble(tk.nextToken());

	// deliver event
	LinkedList l = new LinkedList();
	l.add(event);
	World.receiveEvent(l);
    }

    /**
     * Parse a player param message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parsePlayerParam(String Message, RCWorld World){
	RCPlayerParamEvent event = new RCPlayerParamEvent();
	StringTokenizer tk = new StringTokenizer( Message.substring(PlayerParam.length(), Message.length()-1));

	/* parse event */
	event.Type = PLAYER_PARAM;
	event.player_types = Double.parseDouble(tk.nextToken());
	event.subst_max = Double.parseDouble(tk.nextToken());
	event.pt_max = Double.parseDouble(tk.nextToken());
	event.player_speed_max_delta_min = Double.parseDouble(tk.nextToken());
	event.player_speed_max_delta_max = Double.parseDouble(tk.nextToken());
	event.stamoa_inc_max_delta_factor = Double.parseDouble(tk.nextToken());
	event.player_decay_delta_min = Double.parseDouble(tk.nextToken());
	event.player_decay_delta_max = Double.parseDouble(tk.nextToken());
	event.inertia_moment_delta_factor = Double.parseDouble(tk.nextToken());
	event.dash_power_rarte_delta_min = Double.parseDouble(tk.nextToken());
	event.dash_power_rate_delta_max = Double.parseDouble(tk.nextToken());
	event.player_size_delta_factor = Double.parseDouble(tk.nextToken());
	event.kickable_margin_delta_min = Double.parseDouble(tk.nextToken());
	event.kickable_margin_delta_max = Double.parseDouble(tk.nextToken());
	event.kick_rand_delta_factor = Double.parseDouble(tk.nextToken());
	event.extra_stamina_delta_min = Double.parseDouble(tk.nextToken());
	event.extra_stamina_delta_max = Double.parseDouble(tk.nextToken());
	event.effort_max_delta_factor = Double.parseDouble(tk.nextToken());
	event.effort_min_delta_factor = Double.parseDouble(tk.nextToken());

	// deliver event
	LinkedList l = new LinkedList();
	l.add(event);
	World.receiveEvent(l);
    }

    /**
     * Parse a player type  message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parsePlayerType(String Message, RCWorld World){
	RCPlayerTypeEvent event = new RCPlayerTypeEvent();
	StringTokenizer tk = new StringTokenizer( Message.substring(PlayerType.length(), Message.length()-1));	

	/* parse meassage */
	event.Type = PLAYER_TYPE;
	event.id = Integer.parseInt(tk.nextToken());
	event.player_speed_max = Double.parseDouble(tk.nextToken());
	event.stamina_inc_max = Double.parseDouble(tk.nextToken());
	event.player_decay = Double.parseDouble(tk.nextToken());
	event.inertia_moment = Double.parseDouble(tk.nextToken());
	event.dash_power_rate = Double.parseDouble(tk.nextToken());
	event.player_size = Double.parseDouble(tk.nextToken());
	event.kicakable_margin = Double.parseDouble(tk.nextToken());
	event.kick_rand = Double.parseDouble(tk.nextToken());
	event.extra_stamina = Double.parseDouble(tk.nextToken());
	event.effort_max = Double.parseDouble(tk.nextToken());
	event.effort_min = Double.parseDouble(tk.nextToken());

	// deliver event
	LinkedList l = new LinkedList();
	l.add(event);
	World.receiveEvent(l);
    }
    
    /**
     * Parse a server param message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parseHear(String Message, RCWorld World){
	RCHearEvent event = new RCHearEvent();
	String data = Message.substring(Hear.length(), Message.length()-1);
	StringTokenizer tk = new StringTokenizer(data);

	event.TimeStamp = Integer.parseInt(tk.nextToken());
	String Sender = tk.nextToken();

	/* check for package lose */
	if( OldTimeStamp+1 < event.TimeStamp){
	    World.Log.Debug("lost "+(event.TimeStamp-OldTimeStamp)+" packages("+OldTimeStamp+"/"+event.TimeStamp+")");
	}
	OldTimeStamp = event.TimeStamp;

	/* get sender */
	if( Sender.compareTo("online_coach_left") == 0){
	    event.Sender = LEFT_COACH;
	}else if( Sender.compareTo("online_coach_right") == 0){
	    event.Sender = RIGHT_COACH;
	}else if( Sender.compareTo("referee") == 0){
	    event.Sender = REFEREE;
	}else if( Sender.compareTo("self") == 0 ){
	    event.Sender = SELF;
	}else{
	    event.Sender = DIRECTION;
	    event.Direction = Double.parseDouble(Sender);
	}
	event.Message = tk.nextToken();
	event.Type = HEAR;
	
	// deliver event
	LinkedList l = new LinkedList();
	l.add(event);
	World.receiveEvent(l);
    }
    
    /**
     * Parse a see message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parseSee(String Message, RCWorld World){
	LinkedList l = new LinkedList();
 	StringTokenizer tk = new StringTokenizer( Message.substring(See.length(), Message.length()-1));
	int TimeStamp = Integer.parseInt(tk.nextToken()); // get timestamp
	boolean onlyOneTime = false;
	
// 	World.Log.Log("****************************************");
 	World.Log.Log("Player-"+World.PlayerNumber+" : "+Message);
	    

	// make one event for each object
	int loop = tk.countTokens();
	while( tk.hasMoreElements() ){
	//	for( int rl = 0; rl < loop ; rl++ ){
	    boolean AddToList = true;
	    RCSeeEvent event = new RCSeeEvent();
	    String ID = tk.nextToken(")");
	    String PARAM = tk.nextToken(")");
	    ID = ID.substring(" ((".length(), ID.length()); // remove " (("

	    //	    System.out.println("object = " + ID +" -> " +PARAM );

	    // find opbject
	    StringTokenizer tkId = new StringTokenizer(ID);
	    String TK = tkId.nextToken();
	    //System.out.println("TOKEN = " + TK);
	    switch( (TK.getBytes())[0] ){
	    case 'p':
		{
		    //System.out.println("PLAYER");

		    if( tkId.hasMoreTokens() ){
			// find teamname
			event.TeamName = tkId.nextToken();
			if( event.TeamName.compareTo("\""+this.TeamName+"\"") == 0 ){
			    event.Object = FRIEND;			   
			}else{
			    event.Object = ENEMY;
			}
			if( tkId.hasMoreTokens() ){
			    // get uniformnumber
			    event.UniformNumber = Integer.parseInt(tkId.nextToken());
			    
			    if( tkId.hasMoreTokens() ){
				// is he a goalie
				event.Goalie = Boolean.valueOf(tkId.nextToken()).booleanValue();
			    }
			}else{
			    AddToList = false;
			}
		    }else{
			// undefined player
			AddToList = false;
			event.Object = PLAYER;
		    }
		    break;
		}
	    case 'b':
		{
		    //System.out.println("BALL");

		    event.Object = BALL;
		    break;
		}
	    case 'g':
		{
		    //System.out.println("GOAL");

		    // goal
		    event.Object = GOAL;
		    if( (tkId.nextToken().getBytes())[0] == 'l' ){
			// left
			event.FirstLocation = LEFT;
		    }else{
			// right
			event.FirstLocation = RIGHT;
		    }
		    break;
		}
	    case 'f':
		{
		    //System.out.println("FLAG");
			    
		    // flag
		    event.Object = FLAGS;
		    String pos = tkId.nextToken();
		    
		    switch( (pos.getBytes())[0] ){
		    case 'l':
			{
			    event.FirstLocation = LEFT;
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case '0':
				{
				    event.SecondLocation = CENTER;
				    event.Number = 0;
				    break;
				}
			    case 't':
				{
				    event.SecondLocation = TOP;
				    break;
				}
			    case 'b':
				{
				    event.SecondLocation = BOTTOM;
				    break;
				}
			    }
			    if( tkId.hasMoreTokens()){
				event.Number = Integer.parseInt(tkId.nextToken());
			    }else{
				event.Number = -1;
			    }
			    break;
			}
		    case 'c':
			{
			    if( tkId.hasMoreTokens() == false ){
				event.FirstLocation = CENTER;
				break;
			    }
			    event.FirstLocation = CENTER;
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case 't':
				{
				    event.SecondLocation = TOP;
				    break;
				}
			    case 'b':
				{
				    event.SecondLocation = BOTTOM;
				    break;
				}
			    }
			    break;
			}
		    case 'r':
			{
			    event.FirstLocation = RIGHT;
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case '0':
				{
				    event.SecondLocation = CENTER;
				    event.Number = 0;
				    break;
				}
			    case 't':
				{
				    event.SecondLocation = TOP;
				    break;
				}
			    case 'b':
				{
				    event.SecondLocation = BOTTOM;
				    break;
				}
			    }
			    if( tkId.hasMoreTokens()){
				event.Number = Integer.parseInt(tkId.nextToken());
			    }else{
				event.Number = -1;
			    }
			    break;
			}
		    case 'p': // line
			{
			    event.FirstLocation = LINE;
			    
			    // left or right or center
			    switch(tkId.nextToken().getBytes()[0]){
			    case 'l':
				{
				    event.SecondLocation = LEFT;
				    break;
				}
			    case 'c':
				{
				    event.SecondLocation = CENTER;
				    break;
				}
			    case 'r':
				{
				    event.SecondLocation = RIGHT;
				    break;
				}
			    }
			    
			    // top or bottom
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case 't':
				event.ThirdLocation = TOP;
				break;
			    case 'c':
				event.ThirdLocation = CENTER;
				break;
			    case 'b':
				event.ThirdLocation = BOTTOM;
				break;
			    }
			    break;
			}
		    case 'g': // goal
			{
			    event.FirstLocation = GOAL;
			    // left or right
			    if( (tkId.nextToken().getBytes())[0] == 'l' )
				event.SecondLocation = LEFT;
			    else
				event.SecondLocation = RIGHT;


			    if( tk.hasMoreTokens() == false ){
				event.ThirdLocation = UNDEFINED;
				break;
			    }

			    // top or bottom
 			    if ( (tkId.nextToken().getBytes())[0] == 't' )
 				event.ThirdLocation = TOP;
 			    else
 				event.ThirdLocation = BOTTOM;

			    break;
			}
		    case 't':
			{
			    // top
			    event.FirstLocation = TOP;
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case '0':
				{
				    event.SecondLocation = CENTER;
				    event.Number = 0;
				    break;
				}
			    case 'l':
				{
				    event.SecondLocation = LEFT;
				    break;
				}
			    case 'r':
				{
				    event.SecondLocation = RIGHT;
				    break;
				}
			    }
			    if( tkId.hasMoreTokens()){
				event.Number = Integer.parseInt(tkId.nextToken());
			    }
			    break;
			}
		    case 'b':
			{	    
			    // bottom
			    event.FirstLocation = BOTTOM;
			    switch( (tkId.nextToken().getBytes())[0] ){
			    case '0':
				{
				    event.SecondLocation = CENTER;
				    event.Number = 0;
				    break;
				}
			    case 'l':
				{
				    event.SecondLocation = LEFT;
				    break;
				}
			    case 'r':
				{
				    event.SecondLocation = RIGHT;
				    break;
				}
			    }
			    if( tkId.hasMoreTokens()){
				event.Number = Integer.parseInt(tkId.nextToken());
			    }
			    break;
			}
		    }

		    break;
		}
	    case 'l':
		{
		    // line
		    //System.out.println("LINE");
		    event.Object = LINE;
		    String Pos = tkId.nextToken();
		    //System.out.println("Pos = " + Pos);
		    switch( (Pos.getBytes())[0] ){
		    case 't':			
			event.FirstLocation = TOP;
			break;
		    case 'r':
			event.FirstLocation = RIGHT;
			break;
		    case 'b':
			event.FirstLocation = BOTTOM;
			break;
		    case 'l':
			event.FirstLocation = LEFT;
			break;
		    }
		    break;
		}
	    case 'B':
		{
		    AddToList = false;
		    event.Object = BALL;
		    break;
		}
	    case 'F':
		{
		    AddToList = false;
		    event.Object = FLAGS;
		    break;
		}
	    case 'G':
		{
		    AddToList = false;
		    event.Object = GOAL;
		    break;
		}
	    case 'P':
		{
		    AddToList = false;
		    event.Object = PLAYER;
		    break;
		}
	    }

	    World.Log.Log("Player-"+World.PlayerNumber+" : parse see after object loop");

	    // get metric
	    StringTokenizer tkParam = new StringTokenizer(PARAM);
	    int count = tkParam.countTokens();
	    switch(count){
	    case 1:
		{
		    event.Direction = Double.parseDouble(tkParam.nextToken());
		    break;
		}
	    case 2:
		{		    
		    event.Distance = Double.parseDouble(tkParam.nextToken());
		    event.Direction = Double.parseDouble(tkParam.nextToken());
		    break;
		}
	    case 4:
		{		    
		    event.Distance = Double.parseDouble(tkParam.nextToken());
		    event.Direction = Double.parseDouble(tkParam.nextToken());
		    event.DistChange = Double.parseDouble(tkParam.nextToken());
		    event.DirChange = Double.parseDouble(tkParam.nextToken());
		    break;
		}
	    case 6:
		{		    
		    event.Distance = Double.parseDouble(tkParam.nextToken());
		    event.Direction = Double.parseDouble(tkParam.nextToken());
		    event.DistChange = Double.parseDouble(tkParam.nextToken());
		    event.DirChange = Double.parseDouble(tkParam.nextToken());
		    event.HeadFaceDir = Double.parseDouble(tkParam.nextToken());
		    event.BodyFaceDir = Double.parseDouble(tkParam.nextToken());
		    break;
		}
	    }

	    //	    System.out.println("Distance = " + event.Distance + "  Direction = " + event.Direction );

	    World.Log.Log("Player-"+World.PlayerNumber+" : parse see after metric loop");

	    // add standart values
	    event.TimeStamp = TimeStamp;
	    event.Type = SEE;

	    /* check for package lose */
	    if( (OldTimeStamp+1 < event.TimeStamp) && (onlyOneTime == false) ){
		//		World.Log.Debug("lost "+(event.TimeStamp-OldTimeStamp)+" packages("+OldTimeStamp+"/"+event.TimeStamp+")");
		World.Log.Log("Player-"+World.PlayerNumber+" : lost "+(event.TimeStamp-OldTimeStamp)+" packages("+OldTimeStamp+"/"+event.TimeStamp+")");
		onlyOneTime = true;
	    }
	    OldTimeStamp = event.TimeStamp;
	    

	    // add to list
	    if( AddToList == true ){
		if( (event.Object == FLAGS) || (event.Object == LINE) || (event.Object == GOAL) ){
		    l.addFirst(event);
		}else{
		    l.addLast(event);
		}
	    }
	}
	

	World.Log.Log("Player-"+World.PlayerNumber+" : parse see after object loop");
	
	// deliver events
	World.receiveEvent(l);
    }

    /**
     * Parse a sence body message from the server and deliver the result
     * to the world.
     *
     * @param Message message to parse
     * @param world ref. to world
     */
    public void parseSenseBody(String Message, RCWorld World){
	RCSenceEvent event = new RCSenceEvent();
	event.Type = SENCE;
	
	StringTokenizer tk = new StringTokenizer(Message.substring(SenceBody.length(), Message.length()));
	event.TimeStamp = Integer.parseInt(tk.nextToken());
	
	/* check for package lose */
	if( OldTimeStamp+1 < event.TimeStamp){
	    World.Log.Debug("lost "+(event.TimeStamp-OldTimeStamp)+" packages("+OldTimeStamp+"/"+event.TimeStamp+")");
	}
	OldTimeStamp = event.TimeStamp;

	// view mode 
	String ViewMode = tk.nextToken(")");
	if( ViewMode.indexOf("high") != -1 ){
	    if(ViewMode.indexOf("narrow") != -1){
		event.view_mode = HIGH_NARROW;
	    }else if(ViewMode.indexOf("normal") != -1){
		event.view_mode = HIGH_NORMAL;
	    }else{
		event.view_mode = HIGH_WIDE;
	    }
	}else{
	    if(ViewMode.indexOf("narrow") != -1){
		event.view_mode = LOW_NARROW;
	    }else if(ViewMode.indexOf("normal") != -1){
		event.view_mode = LOW_NORMAL;
	    }else{
		event.view_mode = LOW_WIDE;
	    }	
	}

	// get stamia
	StringTokenizer tkStamia = new StringTokenizer(tk.nextToken().trim());
	tkStamia.nextToken();
	event.Stamia = Double.parseDouble(tkStamia.nextToken());
	event.Effort = Double.parseDouble(tkStamia.nextToken());

	// get speed
	StringTokenizer tkSpeed = new StringTokenizer(tk.nextToken().trim());
 	tkSpeed.nextToken();
 	event.AmmountOfSpeed = Double.parseDouble(tkSpeed.nextToken());
 	event.DirectionOfSpeed = Double.parseDouble(tkSpeed.nextToken()); 


	String Dummy;
	// headangle
	Dummy = tk.nextToken();
	event.HeadAngle = Double.parseDouble(Dummy.substring(" (head_angle".length(), Dummy.length()));

	// get kick 
	Dummy = tk.nextToken();
	event.KickCount = Double.parseDouble(Dummy.substring(" (kick".length(), Dummy.length()));
       
	// get dash
	Dummy = tk.nextToken();
	event.DashCount = Double.parseDouble(Dummy.substring(" (dash".length(), Dummy.length()));

	// get turn
	Dummy = tk.nextToken();
	event.TurnCount = Double.parseDouble(Dummy.substring(" (turn".length(), Dummy.length()));

	// get say
	Dummy = tk.nextToken();
	event.SayCount = Double.parseDouble(Dummy.substring(" (say".length(), Dummy.length()));
	
	// get turn_neck
	Dummy = tk.nextToken();
	event.TurnNeckCount = Double.parseDouble(Dummy.substring(" (turn_neck".length(), Dummy.length()));

	// get catch
	Dummy = tk.nextToken();
	event.CatchCount = Double.parseDouble(Dummy.substring(" (catch".length(), Dummy.length()));

	// get move
	Dummy = tk.nextToken();
	event.MoveCount = Double.parseDouble(Dummy.substring(" (move".length(), Dummy.length()));

	// get turn
	Dummy = tk.nextToken();
	event.ChangeViewCount = Double.parseDouble(Dummy.substring(" (change_view".length(), Dummy.length()));

 	while(tk.hasMoreTokens()){
 	    System.out.println(tk.nextToken(")"));
 	}
	

	// deliver event
	LinkedList l= new LinkedList();
	l.add(event);
	World.receiveEvent(l);
    }
}
