/*
 * $Id: RCPlayer.java,v 1.11 2002/07/06 13:18:43 transier Exp $
 *
 * $Log: RCPlayer.java,v $
 * Revision 1.11  2002/07/06 13:18:43  transier
 * add Goalie support
 *
 * Revision 1.10  2002/07/05 18:34:08  stork
 * update dokumentation
 *
 * Revision 1.9  2002/07/03 17:23:38  stork
 * add some debug messages
 *
 * Revision 1.8  2002/07/02 20:56:19  stork
 * add support for central Log
 *
 * Revision 1.7  2002/06/22 20:15:41  stork
 * add some more dokumentation
 *
 * Revision 1.6  2002/06/07 18:16:58  stork
 * add display support handling
 *
 * Revision 1.5  2002/05/30 15:37:16  stork
 * improve handling when server dies before we
 *
 * Revision 1.4  2002/05/29 21:08:05  stork
 * new config file syntax
 *
 * Revision 1.3  2002/05/25 22:05:35  stork
 * add thread support
 *
 * Revision 1.2  2002/05/21 20:57:14  stork
 * first communicator version
 *
 * Revision 1.1.1.1  2002/05/21 20:32:05  stork
 * empty project
 *
 */

/**
 * The RCPlayer class represents one player. It creates all 
 * all the sub components and put them together.
 */
public class RCPlayer extends Thread{
    /** Associations */
    public RCDisplay Display;
    public RCWorld World;
    public RCBrain Brain;
    public RCCommunicator Communicator;
    public RCLog Log;
    
    /**
     * Constructor for the RCPlayer class
     *
     * @param TeamName name of the the team
     * @param Area area were we should play
     * @param Visual activate world visual.
     * @param Hostname hostname of the server
     * @param Port port of the server
     * @param Goalie are we goale
     */
    public RCPlayer (String TeamName, 
		     boolean Visual, 
		     String Hostname, 
		     int Port, 
		     boolean Goalie,
		     float X1,
		     float Y1,
		     float X2,
		     float Y2,
		     RCLog Log){
	// create a new world
	World = new RCWorld(Log);
	World.X1 = X1;
	World.Y1 = Y1;
	World.X2 = X2;
	World.Y2 = Y2;

	// create a new communicator
	Communicator = new RCCommunicator(TeamName, World, Goalie, Hostname, Port);

	// create a new display
	Display = new RCDisplay(World);
	World.setDisplay(Display);
	setVisible(Visual);

	// create a new brain
	if (Goalie)
		Brain = new RCGoalieBrain(World, Communicator);      
	else
		Brain = new RCBrain(World, Communicator);      
	World.setBrain(Brain);	

	this.Log = Log;
    }

    /**
     * This function is for starting the player(the thread).
     */
    public void run (  ){
	boolean noError = true;
	while( (isInterrupted() == false) && (noError == true) ){
	    Log.Log("Player-"+World.PlayerNumber+" : get a new message ********************");
	    noError = Communicator.receive();
	}
    }

    /**
     * return true if the RCDisplay is visible
     *
     * @return the state of the visualisation
     */
    public boolean isVisible(){
	return Display.isVisible();
    }

    /**
     * activate or deactivate the the visualisation for the player.
     *
     * @param Visible determine the action
     */
    public void setVisible( boolean Visible ){
	this.Display.setVisible( Visible );
    }
}

