/*
 * $Id: RCViewPort.java,v 1.9 2002/07/09 21:42:32 stork Exp $
 *
 * $Log: RCViewPort.java,v $
 * Revision 1.9  2002/07/09 21:42:32  stork
 * update doukumentation
 *
 * Revision 1.8  2002/07/09 18:27:11  oliver
 * # more comments added
 *
 * Revision 1.7  2002/07/07 22:07:27  oliver
 * # faulty line corrected
 *
 * Revision 1.6  2002/07/07 21:57:15  oliver
 * # comments for clipping algorithm added
 * # colors & size of players and ball in Display corrected
 *
 * Revision 1.5  2002/07/04 19:02:20  oliver
 * # clipping algorithm: all objects outside of viewing cone are no longer displayed
 * # debugged display of viewing cone: angle and line length displayed correctly now
 *
 * Revision 1.4  2002/07/02 22:40:15  oliver
 * # Viewing cone of current player
 * # goal line and middle line
 * # all other players visible
 *
 * Revision 1.3  2002/06/23 12:58:12  oliver
 * - RCViewPort inherits from JComponent now --> double buffering -->no flickering
 * - Display of each players quad & the ball
 *
 * Revision 1.2  2002/06/17 15:33:20  oliver
 * Update of RCDisplay & RCViewport; first step of visualisation:
 * Window popup, own player and lines visible
 *
 * Revision 1.1.1.1  2002/05/21 20:32:06  stork
 * empty project
 *
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.math.*;

/**
 * Visual component that display the player's internal world
 */
public class RCViewPort extends JComponent implements RCType
{
    public RCWorld World;
    public int Width;    // default width
    public int Height;   // default height
    public int Scale;    // scaling factor
    public int Border;   // border in pixel


    /**
     *  Constructor 
     **/

    public RCViewPort(RCWorld World)
    {
	this.World = World;
	this.Width = 105;
	this.Height = 68;
	this.Scale = 4;
	this.Border = 5;
    }


    /**
     * overload paint 
     *
     * @param g paint device
     *
     **/
    public void paint( Graphics g )
    {
	// does the actual drawing
	g.setColor(new Color(0,130,0)); // the grass is green
	g.fillRect(0,0, Width*Scale+2*Border,
		        Height*Scale+2*Border);

	// draw the lines
	g.setColor(Color.white);
	g.drawLine(Border, Border, Border+Width*Scale, Border);                          // borderlines
	g.drawLine(Border+Width*Scale, Border, Border+Width*Scale,Border+Height*Scale);
	g.drawLine(Border+Width*Scale, Border+Height*Scale, Border, Border+Height*Scale);
	g.drawLine(Border, Border, Border, Border+Height*Scale);

	g.drawLine( transX(0.0), transY(-34.0), transX(0.0) , transY(34.0) );		// middle line

	g.setColor(Color.black);
	g.drawLine( transX(-52.5), transY(-7.01), transX(-52.5), transY(7.01) );  // goalmouth on the right
	g.drawLine( transX( 52.5), transY(-7.01), transX( 52.5), transY(7.01) );  // goalmouth on the left


	
	

	// draw current player
	RCObject player = World.getOurself();
	g.setColor(Color.blue);
	g.fillOval( transX((World.getOurself()).X(0)),		// "the loonatic is on the grass ..."
		    transY((World.getOurself()).Y(0)), 7, 7);	// (Pink Floyd, Dark Side Tf The Moon)

	// draw current player-sectors
	g.setColor(Color.yellow);
	
	g.drawLine( transX(World.X1), transY(World.Y1), 
		    transX(World.X2), transY(World.Y1) );

	g.drawLine( transX(World.X2), transY(World.Y1), 
		    transX(World.X2), transY(World.Y2) );

  	g.drawLine( transX(World.X1), transY(World.Y2), 
		    transX(World.X2), transY(World.Y2) );

	g.drawLine( transX(World.X1), transY(World.Y1), 
		    transX(World.X1), transY(World.Y2) );	
	


	// draw current player's viewing cone (visible_distance 3.0 & visible_angle 90.0 deg & 
	// team_too_far_length 60.0)

	int selfX = transX((World.getOurself()).X(0));
	int selfY = transY((World.getOurself()).Y(0));

	int linksX=transX((World.getOurself()).X(0) + 60.0*(Math.cos( ((World.angle-45)/180)*Math.PI)));
	int linksY=transY((World.getOurself()).Y(0) + 60.0*(Math.sin( ((World.angle-45)/180)*Math.PI))) ;
	
	g.setColor(Color.blue);
	g.drawLine( selfX, selfY, linksX, linksY );


	int mitteX=transX((World.getOurself()).X(0) + 60.0*(Math.cos( (World.angle/180)*Math.PI )));
	int mitteY=transY((World.getOurself()).Y(0) + 60.0*(Math.sin( (World.angle/180)*Math.PI )));

	g.setColor(Color.red);
	g.drawLine( selfX, selfY, mitteX, mitteY ); 


	int rechtsX=transX((World.getOurself()).X(0) + 60.0*(Math.cos( ((World.angle+45)/180)*Math.PI)));
	int rechtsY=transY((World.getOurself()).Y(0) + 60.0*(Math.sin( ((World.angle+45)/180)*Math.PI)));

	g.setColor(Color.blue);	
	g.drawLine( selfX, selfY, rechtsX, rechtsY ); 
	
	


	// draw the ball if within visible area

	g.setColor(Color.white);

	if ( in( transX(World.Ball.X(0)), transY(World.Ball.Y(0)),
		 selfX  , selfY,
		 linksX , linksY,
		 mitteX , mitteY,
		 rechtsX, rechtsY ) == true )
	    { g.fillOval(transX(World.Ball.X(0)), transY(World.Ball.Y(0)), 7, 7); }



	// draw the enemy if within visible area
	
	g.setColor(Color.red);
	for (int x = 0; x < 11; x++)
	    {	
		if ( in( transX(World.Enemies[x].X(0)), transY(World.Enemies[x].Y(0)),
		     selfX  , selfY,
		     linksX , linksY,
		     mitteX , mitteY,
		     rechtsX, rechtsY ) == true )
		{   g.fillOval(transX(World.Enemies[x].X(0)), transY(World.Enemies[x].Y(0)), 7, 7); }
	    }



	// draw the friends if within visible area

	g.setColor(Color.blue);
	
	for (int x = 0; x < 11; x++)
	    {
		if ( in( transX(World.Friends[x].X(0)), transY(World.Friends[x].Y(0)),
		     selfX  , selfY,
		     linksX , linksY,
		     mitteX , mitteY,
		     rechtsX, rechtsY ) == true )
		{ g.fillOval(transX(World.Friends[x].X(0)), transY(World.Friends[x].Y(0)), 7, 7); }
	    }
    }




    /**
     * check the orientation of 3 given points p0, p1 and p2: 
     * check whether we are moving counter-clock-wise if we go from p0 to p1 to p2 or NOT.
     * 
     * @param x0 x-coord.s of point p0 
     * @param y0 y-coord.s of point p0
     * @param x1 x-coord.s of point p1
     * @param y1 y-coord.s of point p1
     * @param x2 x-coord.s of point p2
     * @param y2 y-coord.s of point p2
     * 
     * @return -1  if points are collinear & p0 in between p1 and p2 
     * @return  0  if p2 is in between p0 and p1 
     *	        1  if p1 is in between p0 and p2
     *
     **/

    public int ccw( int x0, int y0,
		    int x1, int y1,
		    int x2, int y2 )
    {
	int dx1, dx2, dy1, dy2;
	
	dx1 = x1 - x0;
	dy1 = y1 - y0;
	dx2 = x2 - x0;
	dy2 = y2 - y0;
	
	if ( dx1*dy2 > dy1*dx2 ) return +1;   
	if ( dx1*dy2 < dy1*dx2 ) return -1;  
	
	if ((dx1*dx2 < 0) || (dy1*dy2 < 0))		return -1;
	if ((dx1*dx1 + dy1*dy1) < (dx2*dx2 + dy2*dy2)) 	return +1;
	
	return 0;
    }



    /**
     * Checks whether the two given lines cross on screen:
     * Are the ending points of one line on different sides of the other line?
     * (that means: do they have different ccw-values?)
     *
     * @param l1_x1  x-coord. of starting point of first line
     * @param l1_y1  y-coord. of starting point of first line 
     * @param l1_x2  x-coord. of end point of first line 
     * @param l1_y2  y-coord. of end point of first line
     * @param l2_x1  x-coord. of starting point of second line 
     * @param L2_y1  y-coord. of starting point of second line
     * @param l2_x2  x-coord. of end poitn of second line
     * @param l2_y2  y-coord. of end point of second line
     *
     * @return true   the lines intersect / 
     *         false  the lines do not intersect
     *
     **/

    public boolean intersect( int l1_x1, int l1_y1, 
			      int l1_x2, int l1_y2,
			      int l2_x1, int l2_y1,
			      int l2_x2, int l2_y2 )
    {
	return ((( ccw(l1_x1, l1_y1,
	    	       l1_x2, l1_y2,
		       l2_x1, l2_y1 ) * ccw(l1_x1, l1_y1,
		    			    l1_x2, l1_y2,					    
					    l2_x2, l2_y2)) <= 0) &&
	       (( ccw(l2_x1, l2_y1,
	    	      l2_x2, l2_y2,
		      l1_x1, l1_y1 ) * ccw(l2_x1, l2_y1,
		    			   l2_x2, l2_y2,
					   l1_x2, l1_y2)) <= 0));
    }
    
        
    
    /**
     * checks whether a given point (e.g. an RCObject (Player, Enemy, Ball))  is being
     * contained by the given 4-point-polygon
     *
     * @param  x     x-coordinate of concerned point/RCObject 
     * @param  y     y-coordinate of concerned point/RCObject  
     * @param  x0    x-coord of first vertice of polygon 
     * @param  y0    y-coord of first vertice of polygon 
     * @param  x1    x-coord of second vertice of polygon 
     * @param  y1    y-coord of second vertice of polygon 
     * @param  x2    x-coord of third vertice of polygon 
     * @param  y2    y-coord of third vertice of polygon 
     * @param  x3    x-coord of fourth vertice of polygon 
     * @param  y3    y-coord of fourth vertice of polygon 
     *
     **/

    public boolean in(int x , int y ,  //  Is THIS given point within the polygon defined by ...
		      int x0, int y0,  //
		      int x1, int y1,  //  
		      int x2, int y2,  //
		      int x3, int y3 ) //  ... these vertices?
		      	 
    {
	int count=0;
    
	if (intersect(x, y, 0, -34,  x0, y0, x1, y1)) count++;
	if (intersect(x, y, 0, -34,  x1, y1, x2, y2)) count++;
	if (intersect(x, y, 0, -34,  x2, y2, x3, y3)) count++;
	if (intersect(x, y, 0, -34,  x3, y3, x0, y0)) count++;
    
	if ( count%2 != 0) return true;     // odd number of crossing points: point is within!
	else               return false;    // otherwise it isn't.
    }
    




    /**
     * tell our preferred size to anybody who's interested
     *
     * @return our size
     **/
    public Dimension getPreferredSize()
    {
	return new Dimension(Width*Scale+2*Border,
			     Height*Scale+2*Border);
    }


    /**
     * tell our minimum size to anybody how's interested
     **/
    public Dimension getMinimumSize()
    {
	return new Dimension(Width*Scale+2*Border,
			     Height*Scale+2*Border);
    }


    /**
     * tell actual size to anyone how's interested
     **/
    public Dimension getSize()
    {
	return new Dimension(Width*Scale+2*Border,
			     Height*Scale+2*Border);
    }



    /**
     *  translate world coordinates to image coordiates
     *
     * @param x     : x-value to translate
     * @return no. 2 : the corresponding image value
     */
    public int transX( double x )
    {
	if( World.Side == LEFT )
	    {
//		return Math.abs((int)((Border+Scale*Width/2.0)+Scale*x));
		return (int)((Border+Scale*Width/2.0)+Scale*x);
	    }
	else
	    {
//		return Math.abs((int)((Border+Scale*Width/2.0)-Scale*x));
		return (int)((Border+Scale*Width/2.0)-Scale*x);
	    }
    }      



    /** 
     * now translate world coordinates to image coordinates
     *
     * @param y     y-value to translate
     * @return       the corresponding image value
     **/
    public int transY(double y)
    {
	if (World.Side == LEFT)
	    {
//		return Math.abs((int)((Border+Scale*Height/2.0)+Scale*y));
		return (int)((Border+Scale*Height/2.0)+Scale*y);
	    }
	else
	    {
//		return Math.abs((int)((Border+Scale*Height/2.0)-Scale*y));
		return (int)((Border+Scale*Height/2.0)-Scale*y);
	    }
    }
}

