/*
 * $Id: RCWorld.java,v 1.24 2002/07/05 18:34:08 stork Exp $
 *
 * $Log: RCWorld.java,v $
 * Revision 1.24  2002/07/05 18:34:08  stork
 * update dokumentation
 *
 * Revision 1.23  2002/07/04 16:25:48  stork
 * reactivate log comments 2
 *
 * Revision 1.22  2002/07/04 16:19:13  stork
 * reactivate log comments
 *
 * Revision 1.20  2002/07/04 12:51:53  stork
 * add new data
 *
 * Revision 1.19  2002/07/04 12:47:41  stork
 * add new data
 *
 * Revision 1.18  2002/07/03 19:01:06  stork
 * cleanup
 *
 * Revision 1.17  2002/07/03 17:24:37  stork
 * add some debug Messages
 *
 * Revision 1.15  2002/07/02 20:52:26  stork
 * +invert NewData
 * +RCLog update
 *
 * Revision 1.14  2002/06/27 20:26:40  stork
 * add + update dokumentation
 *
 * Revision 1.13  2002/06/23 21:09:09  stork
 * add support for OutOfField behavior of the player
 *
 * Revision 1.12  2002/06/22 20:14:27  stork
 * add MessagFilter for getting the right playmode
 * add some more dokumentation
 *
 * Revision 1.11  2002/06/21 19:46:08  stork
 * add support for asnc. event handling
 *
 * Revision 1.10  2002/06/10 20:53:14  stork
 * + add calculation of enemies and friend
 * + only update when there was a sence event
 *
 * Revision 1.9  2002/06/08 15:26:51  stork
 * tipfehler
 *
 * Revision 1.8  2002/06/08 15:16:05  stork
 * activate brain
 *
 * Revision 1.7  2002/06/08 14:42:07  stork
 * first version that maybe will work
 *
 * Revision 1.6  2002/06/07 21:35:19  stork
 * maybe working for the left side
 *
 * Revision 1.5  2002/06/07 18:15:45  stork
 * add basic calculation stuff
 *
 * Revision 1.3  2002/05/29 15:18:40  stork
 * update interface and some event hanling methods
 *
 * Revision 1.2  2002/05/21 20:57:14  stork
 * first communicator version
 *
 * Revision 1.1.1.1  2002/05/21 20:32:06  stork
 * empty project
 *
 */
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.math.*;


/**
 * This class is the main component of the Player. This class
 * analyse the server messages and calculates the for all objects
 * on the field.
 */
public class RCWorld  
    implements RCPlayMode, RCView, RCType, RCSender, RCEventType {
    /** Attributes */
    public RCObject Ball;       // the ball
    public RCObject[] Enemies;  // the enemies
    public RCObject[] Friends;
    public RCEvent Event;
    public boolean NewEvent;
    public int PlayerNumber;    

    // defines the quad. of the player
    public double X1;
    public double Y1;
    public double X2;
    public double Y2;
    
    public double angle; // abs angle of the player
    public double AngleToBall;        // contains the relative angle from angle(RCWorld) to the ball
    public int AngleToBallTimeStamp;  // timestamp of the value in AngleToBall
    public boolean NewData;           // true if every

    public final String MsgStart = "RCTEAM:"; // prefix for our messages 

    public int GameMode;  //  current game mode

    public int Side;    // our side

    public RCServerParamEvent ServerParam;
    public RCPlayerParamEvent PlayerParam;
    public RCPlayerTypeEvent PlayerType[];
    public RCSenceEvent SenceBody;

    /* little helpers for calculation */
    public RCSeeEvent NearestFlag = null;
    public RCSeeEvent NearestLine = null;
    public boolean ValidPosition = false;

    public int CurrentTimeStamp; // the 

    /* depreciated helpers for the handling position out of the field */
    boolean OutOfField;
    int OutOfFieldCounter;
    boolean enableOutOfField;
    Point2D.Double OldPosition;

    /** Associations */
    private RCDisplay Display;
    private RCBrain Brain;
    public RCLog Log;
    
    /**
     * Constructor. Initialize a new object.
     */
    public RCWorld ( RCLog Log ){
	/* init */
	CurrentTimeStamp = -1;
	Friends = new RCObject[11];
	Enemies = new RCObject[11];
	Ball = new RCObject();
	PlayerType = new RCPlayerTypeEvent[8];
	Brain = null;
	Display = null;
	this.Log = Log;
	OutOfField = false;
	enableOutOfField = false;
	OldPosition = new Point2D.Double();
	OldPosition.x = 0;
	OldPosition.y = 0;

	// create objects
	for( int x = 0; x < 11 ; x++){
	    Friends[x] = new RCObject();
	    Enemies[x] = new RCObject();
	}	
    }

    
    /**
     * Sets the strategy component of the player.
     *
     * @param Brain ref. to a RCBrain component
     */
    public void setBrain( RCBrain Brain ){
	this.Brain = Brain;
    }

    
    /**
     * set the visualisation component of the player
     *
     * @param Display ref. to the vis. component
     */
    public void setDisplay( RCDisplay Display ){
	this.Display = Display;
    }

    /**
     * This function get a list full of RCEvent* 's. The function remove and process every event 
     * in the list. After the processing the function update the strategy and the display component.
     *
     * @param Event list with events
     */
    public synchronized void receiveEvent ( LinkedList Events ){
	NearestFlag = null;
	NearestLine = null;
	ValidPosition = false;
	boolean update = false;
	OutOfFieldCounter = 0;

	Log.Log("Player-"+PlayerNumber+" : CurrentTimeStamp "+CurrentTimeStamp);

	/* process the event list */
	try{
	    ListIterator it = Events.listIterator(0);
	    while( it.hasNext() ){
		RCEvent event = (RCEvent)it.next();
		it.remove();
		switch( event.Type ){
		case SERVER_PARAM:
		    {
			/* backup the ServerParamEvent */
			Log.Log("Player-"+PlayerNumber+" : work on ServerParamMessage");
			this.ServerParam = (RCServerParamEvent)event;
			break;
		    }
		case PLAYER_PARAM:
		    {
			/* backup the PlayerParamEvent */
			Log.Log("Player-"+PlayerNumber+" : work on PlayerParam-Message");
			this.PlayerParam = (RCPlayerParamEvent)event;
			break;
		    }
		case PLAYER_TYPE:
		    {
			/* backup the PlayerTypeEvent */
			Log.Log("Player-"+PlayerNumber+" : work on PlayerType-Message");
			RCPlayerTypeEvent PlayerTypeEvent = (RCPlayerTypeEvent)event;
			this.PlayerType[PlayerTypeEvent.id] = PlayerTypeEvent;
			break;
		    }
		case HEAR:
		    {
			/* parse the HearMessage */
			Log.Log("Player-"+PlayerNumber+" : work on Hear-Message");
			this.MessageFilter((RCHearEvent)event);
			break;	 
		    }
		case SENCE:
		    {
			/* backup SenceBodyEvent */
			Log.Log("Player-"+PlayerNumber+" : work on SenceBody-Message");
			this.SenceBody = (RCSenceEvent)event;
			update = true;
			break;
		    }
		case SEE:
		    {		       
			Log.Log("Player-"+PlayerNumber+" : work on See-Message");
			CurrentTimeStamp = ((RCSeeEvent)event).TimeStamp;
			NewData = true; // we got freshmeat
			//update = true;
			
 			// find nearest line + flag
			if( (ValidPosition == false ) &&
			    ((((RCSeeEvent)event).Object == FLAGS ) ||
			     (((RCSeeEvent)event).Object == GOAL )  ||
			     (((RCSeeEvent)event).Object == LINE )) ){
			    
			    if( (((RCSeeEvent)event).Object == FLAGS) ||
				(((RCSeeEvent)event).Object == GOAL )){
				// flags
				if( NearestFlag == null ){
				    NearestFlag = (RCSeeEvent)event;
				    break;
				}else{
				    if( NearestFlag.Distance >= (((RCSeeEvent)event).Distance) ){
					NearestFlag = (RCSeeEvent)event;
					break;
				    }
				}
			    }else if( ((RCSeeEvent)event).Object == LINE ){ // LINE
				// line
				if( NearestLine == null ){
				    NearestLine = (RCSeeEvent)event;
				    break;
				}else{
				    if( NearestLine.Distance >= (((RCSeeEvent)event).Distance) ){
					NearestLine = (RCSeeEvent)event;
					break;
				    }
				}
			    }
			    break;			    
			}
			
			// calculate our position
			if( ValidPosition == false ){
			    calculatePosition();
			}
			

			double AbsAngle = 0;
			double d = 0;
			
			// process the other events
 			switch( ((RCSeeEvent)event).Object ){			 
 			case BALL:
 			    AbsAngle = ((angle + ((RCSeeEvent)event).Direction)/180)*Math.PI;
 			    d = ((RCSeeEvent)event).Distance;
  			    Ball.insertTime(((RCSeeEvent)event).TimeStamp );
  			    Ball.insertX(getOurself().X(0) + d*Math.cos(AbsAngle));
  			    Ball.insertY(getOurself().Y(0) + d*Math.sin(AbsAngle));	
			    AngleToBall =  ((RCSeeEvent)event).Direction;
			    AngleToBallTimeStamp = ((RCSeeEvent)event).TimeStamp;
 			    break;
 			case ENEMY:
			    if( (((RCSeeEvent)event).UniformNumber > 0) &&
				(((RCSeeEvent)event).UniformNumber <= 11)){
				int Number = ((RCSeeEvent)event).UniformNumber;
				AbsAngle = ((angle + ((RCSeeEvent)event).Direction)/180)*Math.PI;
				d = ((RCSeeEvent)event).Distance;
				Enemies[Number-1].insertTime(((RCSeeEvent)event).TimeStamp );
				Enemies[Number-1].insertX(getOurself().X(0) + d*Math.cos(AbsAngle));
				Enemies[Number-1].insertY(getOurself().Y(0) + d*Math.sin(AbsAngle));   	
			    }
 			    break;
 			case FRIEND:
			    if( (((RCSeeEvent)event).UniformNumber > 0) &&
				(((RCSeeEvent)event).UniformNumber <= 11)){
				int Number = ((RCSeeEvent)event).UniformNumber;
				AbsAngle = ((angle + ((RCSeeEvent)event).Direction)/180)*Math.PI;
				d = ((RCSeeEvent)event).Distance;
				Friends[Number-1].insertTime(((RCSeeEvent)event).TimeStamp );
				Friends[Number-1].insertX(getOurself().X(0) + d*Math.cos(AbsAngle));
				Friends[Number-1].insertY(getOurself().Y(0) + d*Math.sin(AbsAngle));   	
			    }
 			    break;
 			}
			
			break;
		    }
		}		
	    }


	    // no objects out there
	    if( ValidPosition == false )
		calculatePosition();

	    // update only when there was an sence event
	    if( update == true ){
	
		// lets think a little bit
		if( (Brain != null) ){
		    int x = 0;
		    Log.Log("Player-"+PlayerNumber+" : enter brain");
		    //NewData = !NewData;
		    x = Brain.update(); 
		    //NewData = !NewData;
		    Log.Log("Player-"+PlayerNumber+" : brain return "+x);
		    Log.Log("Player-"+PlayerNumber+" : leave brain");
		    NewData = false;  // dont use the old data in the second call
		}

		/* update display */
		if( (Display != null) && (getOurself().Time(0)%10 == 0)){
		    Display.update();
		}


		//Log.Log("Angele = " + angle +" / "+AngleToBall);
	
	    }
   
	}catch(IndexOutOfBoundsException e){
	    // shit happens
	    Log.Debug("Error pos "+e);
	    //e.printStackTrace();
	    return;
	}
    }


    /* 
     * This function calculate the position of our player.
     */
    public void calculatePosition(){
	double x = 0;
	double y = 0;
	if( (NearestLine == null) || (NearestFlag == null) ){
	    return; // leave 
	}else{
				// calculate abs angle 
	    if( Side == LEFT){
		// calculation for left side
				    
		//calculate abs angle
		switch(NearestLine.FirstLocation){
		case TOP:
		    /* calculation for top line */
		    if( NearestLine.Direction > 0 )
			angle = -1*NearestLine.Direction;
		    else
			angle =  -90 - (90+ NearestLine.Direction);
		    if( OutOfField == true )
			angle += 180;
		    break;
		case LEFT:
		    /* calculation for left line */
		    if(NearestLine.Direction < 0){
			angle = 180  - (90+NearestLine.Direction);
			if( OutOfField == true )
			    angle += 180;
		    }else{
			angle = -90 -NearestLine.Direction;
			if( OutOfField == true )
			    angle -= 180;
		    }
		    break;
		case RIGHT:
		    /* calculation for right line */
		    if( NearestLine.Direction < 0 ){
			angle = -1*(90+NearestLine.Direction);
			if( OutOfField == true )
			    angle -= 180;
		    }else{
			angle = 90 - NearestLine.Direction;
			if( OutOfField == true )
			    angle += 180;
		    }
		    break;
		case BOTTOM:
		    /* calculation for bottom line */
		    if( NearestLine.Direction > 0 )
			angle = 180 - NearestLine.Direction;
		    else
			angle = -1 * NearestLine.Direction;
		    if( OutOfField == true )
			angle -= 180;
		    break;
		}
	    }else{
		// calculation for left side2
				    
		//calculate abs angle 
		switch(NearestLine.FirstLocation){
		case TOP:
		    /* calculation for top line */
		    if( NearestLine.Direction < 0){
			angle = -1* NearestLine.Direction;
		    }else{
			angle = 180 - NearestLine.Direction;
		    }
		    if( OutOfField == true )
			angle -= 180;
		    break;
		case LEFT:
		    /* calculation for left line */
		    if( NearestLine.Direction < 0){
			angle = -1*(90 + NearestLine.Direction) ;
			if( OutOfField == true )
			    angle -= 180;
		    }else{
			angle = 90-NearestLine.Direction;
			if( OutOfField == true )
			    angle += 180;
		    }
		    break;
		case RIGHT:
		    /* calculation for right line */
		    if( NearestLine.Direction < 0){
			angle = 180 - (90+NearestLine.Direction);
			if( OutOfField == true )
			    angle += 180;
		    }else{
			angle = -90 - NearestLine.Direction; 
			if( OutOfField == true )
			    angle -= 180;
		    }
		    break;
		case BOTTOM:
		    /* calculation for bottom line */
		    if( NearestLine.Direction < 0){
			angle = -1*(180 + NearestLine.Direction);
		    }else{
			angle = -1*NearestLine.Direction;
		    }
		    if( OutOfField == true )
			angle += 180;
		    break;
		}
	    }

	    // check in/out of field


	    // calculate position 
	    Point2D.Double Flag = getFlag( (RCSeeEvent)NearestFlag );
	    
	    if( Side == RIGHT ){
		Flag.x *= -1;
		Flag.y *= -1;
	    }				

//  	    Log.Log("angle = " + angle );
//  	    Log.Log("Flag.x = " + Flag.getX() + "   Flag.y = " + Flag.getY());
//  	    Log.Log("Distance = "+ NearestFlag.Distance );

	    // calculate and insert values
	    double AbsAngle = angle + NearestFlag.Direction;
// 	    getOurself().insertX(((double)Flag.getX()) - NearestFlag.Distance*Math.cos((AbsAngle/180.0)*Math.PI));
// 	    getOurself().insertY(((double)Flag.getY()) - NearestFlag.Distance*Math.sin((AbsAngle/180.0)*Math.PI));
// 	    getOurself().insertTime(NearestLine.TimeStamp);

	    x = (((double)Flag.getX()) - NearestFlag.Distance*Math.cos((AbsAngle/180.0)*Math.PI));
	    y = (((double)Flag.getY()) - NearestFlag.Distance*Math.sin((AbsAngle/180.0)*Math.PI));
	    getOurself().insertTime(NearestLine.TimeStamp);


	    // check and maybe recalculate
	    if( (CurrentTimeStamp >= 10) &&
		(enableOutOfField == true) &&
	        ((Math.abs( OldPosition.x - x ) > 5) ||(Math.abs( OldPosition.y - y ) > 5))){
		OutOfField = !OutOfField;
		Log.Log("Player "+ PlayerNumber + " : OUT OF FIELD  switch "+ OutOfField);
		
		// check for moving
		if( ++OutOfFieldCounter >= 3 ){
		    // calculate new position
		    enableOutOfField = false; // deactivate
		    calculatePosition();      // recalculate a position in the field
		    enableOutOfField = true;  // reactivate
		}else{
		    calculatePosition();  // recalculate 
		    return;               // return 
		}
	    }
	    
	    // set the values
	    getOurself().insertX(x);
	    getOurself().insertY(y);
	    getOurself().insertTime(NearestLine.TimeStamp);
	    OldPosition.x = x;
	    OldPosition.y = y;

//  	    Log.Log("("+getOurself().X(0)+"/"+getOurself().Y(0)+")");
	    
				
	    // we got our position 
	    ValidPosition = true;
	}

    }


    /**
     * get a handle of the current player
     *
     * @return ref. to this player 
     */
    public RCObject getOurself(){
	try{
	    return Friends[PlayerNumber-1];
	}catch(IndexOutOfBoundsException e){
	    return new RCObject();
	}
    }


    /**
     * find and return the abs coordinates for the given flag and left side
     * if you want the coordinates negate x and y
     *
     * @param event Flag-RCSeeEvent
     * @return Point
     */
    public Point2D.Double getFlag(RCSeeEvent event){
	Point2D.Double Flag = new Point2D.Double(0,0);
	
	/* we see the goal */
	if( event.Object == GOAL ){
	    Flag.y = 0;
	    if( event.FirstLocation == LEFT )
		Flag.x = -52.5;
	    else
		Flag.x = 52.5;
	    
	    return Flag;
	}

	switch( event.FirstLocation ){
	case TOP:
	    Flag.y = -39;
	    switch( event.SecondLocation ){
	    case LEFT:
		Flag.x = -1*event.Number;
		break;
	    case CENTER:
		Flag.x = 0;
		break;
	    case RIGHT:
		Flag.x = event.Number;
		break;
	    }
	    return Flag;
	case RIGHT:
	    Flag.x = 57.5;
	    switch( event.SecondLocation ){
	    case TOP:
		if( event.Number < 0 ){
		    Flag.x = 52.5;
		    Flag.y = -34;
		}else{
		    Flag.y = -1*event.Number;
		}
		break;
	    case CENTER:
		Flag.y = 0;
		break;
	    case BOTTOM:
		if( event.Number < 0){
		    Flag.x = 52.5 ;
		    Flag.y = 34;
		}else{
		    Flag.y = event.Number;
		}
		break;
	    }
	    return Flag;
	case BOTTOM:
	    Flag.y = 39;
	    switch( event.SecondLocation ){
	    case LEFT:
		Flag.x = -1*event.Number;
		break;
	    case CENTER:
		Flag.x = 0;
		break;
	    case RIGHT:
		Flag.x = event.Number;
		break;
	    }
	    return Flag;
	case LEFT:
	    Flag.x = -57.5;
	    switch( event.SecondLocation ){
	    case TOP:
		if( event.Number < 0 ){
		    Flag.y = -34;
		    Flag.x = -52.5;
		}else{
		    Flag.y = -1*event.Number;
		}
		break;
	    case CENTER:
		Flag.y = 0;
		break;
	    case BOTTOM:
		if( event.Number < 0 ){
		    Flag.y = 34;
		    Flag.x = -52.5;
		}else{
		    Flag.y = event.Number;
		}
		break;
	    }
	    return Flag;
	case GOAL:
	    Flag.x = -52.5;
	    switch( event.ThirdLocation ){
	    case TOP:
		Flag.y = -7.01;
		break;
	    case BOTTOM:
		Flag.y = 7.01;
		break;
	    case UNDEFINED:
		Flag.y = 0;
		break;
	    }
	    
	    if( event.SecondLocation == RIGHT )
		Flag.x *= -1;

	    return Flag;	    
	    //	    break;
	case LINE:   // (l p  ( line before the goals
	    switch( event.ThirdLocation ){
	    case TOP:
		Flag.x = -36;
		Flag.y = -20.175;
		break;
	    case CENTER:
		Flag.x = -36;
		Flag.y = 0;
		break;
	    case BOTTOM:
		Flag.x = -36;
		Flag.y = 20.175;
		break;
	    }
	    if( event.SecondLocation == RIGHT ){
		Flag.x *= -1;
	    }
	    return Flag;
	case CENTER:
	    if( event.SecondLocation == UNDEFINED ){
		// (f c)
		return new Point2D.Double(0, 0);
	    }else if( event.SecondLocation == TOP ){
		// (f c t)
		return new Point2D.Double(0, -34);
	    }else{
		// (f c b)
		return new Point2D.Double(0, 34);		
	    }	    
	}

	return new Point2D.Double(0,0);
    }

    /**
     * analyse and filter messages from server and set the corresponding 
     * playmode.
     *
     * @param Event the event to filter
     */
    public void MessageFilter(RCHearEvent Event){
	if( Event.Message.startsWith( MsgStart ) == true ){
	    // get our messages
	    // at the moment we don't use this feature
	}else if( Event.Message.startsWith("before") == true ){
	    GameMode = BEFORE_KICKOFF;
	}else if( (Event.Message.compareTo("play_on") == 0) ){
	    GameMode = PLAY_ON;
	}else if( (Event.Message.compareTo("time_over") == 0) ){
	    GameMode = TIME_OVER;
	}else if( (Event.Message.compareTo("kick_off_l") == 0) ){
	    GameMode = KICKOFF_LEFT;
	}else if( (Event.Message.compareTo("kick_off_r") == 0) ){
	    GameMode = KICKOFF_RIGHT;
	}else if( (Event.Message.compareTo("kick_in_l") == 0) ){
	    GameMode = KICKIN_LEFT;
	}else if( (Event.Message.compareTo("kick_in_r") == 0) ){
	    GameMode = KICKIN_RIGHT;
	}else if( (Event.Message.compareTo("free_kick_l") == 0) ){
	    GameMode = FREEKICK_LEFT;
	}else if( (Event.Message.compareTo("free_kick_r") == 0) ){
	    GameMode = FREEKICK_RIGHT;
	}else if( (Event.Message.compareTo("corner_kick_l") == 0) ){
	    GameMode = CORNERKICK_LEFT;
	}else if( (Event.Message.compareTo("corner_kick_r") == 0) ){
	    GameMode = CORNERKICK_RIGHT;
	}else if( (Event.Message.compareTo("goal_kick_l") == 0) ){
	    GameMode = GOALKICK_LEFT;
	}else if( (Event.Message.compareTo("goal_kick_r") == 0) ){
	    GameMode = GOALKICK_RIGHT;
	}else if( Event.Message.startsWith("goal_l") ){
	    GameMode = KICKOFF_RIGHT;
	}else if( Event.Message.startsWith("goal_r") ){
	    GameMode = KICKOFF_RIGHT;
	}else if( (Event.Message.compareTo("offside_l") == 0) ){
	    GameMode = FREEKICK_RIGHT;
	}else if( (Event.Message.compareTo("offside_r") == 0) ){
	    GameMode = FREEKICK_LEFT;
	}else if( (Event.Message.compareTo("drop_ball") == 0) ){
	    GameMode = PLAY_ON;
	}else if( (Event.Message.compareTo("foul_l") == 0) ){
	    GameMode = FREEKICK_RIGHT;
	}else if( (Event.Message.compareTo("foul_r") == 0) ){
	    GameMode = FREEKICK_LEFT;
	}else if( (Event.Message.compareTo("goalie_catch_ball_l") == 0) ){
	    GameMode = FREEKICK_LEFT;
	}else if( (Event.Message.compareTo("goalie_catch_ball_r") == 0) ){
	    GameMode = FREEKICK_RIGHT;
	}else if( Event.Message.startsWith("time_up") ){
	    GameMode = TIME_OVER;
	}else if( (Event.Message.compareTo("half_time") == 0) ){
	    GameMode = BEFORE_KICKOFF;
	}else if( (Event.Message.compareTo("time_extended") == 0) ){
	    GameMode = BEFORE_KICKOFF;
	}
    }
}

